/* Copyright 2008-2017 Research Foundation State University of New York   */

/* This file is part of QUB Online.                                       */

/* QUB Online is free software; you can redistribute it and/or modify    */
/* it under the terms of the GNU General Public License as published by   */
/* the Free Software Foundation, either version 3 of the License, or      */
/* (at your option) any later version.                                    */

/* QUB Online is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License,     */
/* named LICENSE.txt, in the QUB Online program directory.  If not, see  */
/* <http://www.gnu.org/licenses/>.                                        */

#ifndef QUBX_SIMULATE_H
#define QUBX_SIMULATE_H

#ifndef EMSCRIPTEN
 #include "qubfast.h"
#endif

#include "qubx_model.h"

// threshold of Nchannel above which no idealization is generated:
#define QUBX_SIMULATE_MANY 16

// will use A matrix method if any -Q(i,i) [exit rate sum] > QUBX_SIMULATE_A_THRESHOLD*sampling (Hz)
// (A is slower unless rates are near sampling in a way that results in short events)
#define QUBX_SIMULATE_A_THRESHOLD 0.5

// minimum number of channels for a single segment to be split and multithreaded
#define QUBX_SIMULATE_SPLIT_THRESHOLD 16
// and max samples
#define QUBX_SIMULATE_SPLIT_MAXPOINTS 1000000


#ifdef __cplusplus
extern "C" {
#endif

QUBFAST_API int    qubx_simulate1(qubx_model *model, int *start_states, int add_baseline, int use_Peq, int v_signal, float v_def,
				 double sampling, int Nstimulus, int *stim_counts, float **stimuli,
				 int Nsample, float *samples, int *states, int *counts, double *amps, double *stds,
				 int force_Q, int seed);
#ifndef EMSCRIPTEN
QUBFAST_API int    qubx_simulate(qubx_model *model, int *start_states, int add_baseline, int use_Peq, int v_signal, float v_def,
				 double sampling, int Nstimulus, int Nseg, int **stim_counts, float ***stimuli,
				 int *Nsample, float **samples, int *ndwt, int **states, int **counts, double **amps, double **stds,
				 int force_Q, StatusCallback pctCB, void *pctObj);
#endif
/*
Simulates sampled and state-idealized HMM data and returns the number of dwells written into states and counts.

    @param sampling: interval between samples, in seconds
    @param nchannel: number of identical mechanisms' responses to sum together
    @param nstate: number of model states (vertices)
    @param start_states: array[nstate] of entry state indices, or -1 to pick fresh.  On output, contains exit states.
    @param nsample: number of data points to return in samples
    @param nstimulus: number of stimulus signals, plus signal 0 which is ignored (so at least 1)
    @param stim_counts: number of datapoints in each signal; -1 if it's constant
    @param stimuli: array[max(1,stim_counts[i])] of datapoints for each signal
    @param v_signal: index of Voltage signal, if exc_amp is to be scaled by 1e-3*(Voltage-vRev)
    @param samples: output array[nsample] for data points
    @param states: output array[nsample] for idealized state index of each dwell
    @param counts: output array[nsample] for number of samples in each dwell

    @param pctCB: callback pctCB(pctObj, complete), 0.0 <= complete <= 1.0; Return nonzero to stop the simulation.
    @param pctObj: data pointer for the callback function, or NULL
  
 */


QUBFAST_API int  qubx_simulate_make_idealized_flc(int *ff, int *ll, int *cc, int N, int *clazz, int *states, int *st_counts, int at);



QUBFAST_API void qubx_simulate_baselines(int Nsegment, int *Nsamples, float **sample_arrays, double sampling,
					 int baselineFluct, int baselineFluctN, double baselineFluctLifetime, double baselineFluctMaxAmp,
					 int baselineDrift, double baselineDriftSlope, double baselineDriftIntercept,
					 int baselinePeriodic, double baselinePeriodicFreq, double baselinePeriodicPhase, double baselinePeriodicAmp,
					 int baselineJumps, double baselineJumpsAmpStd,
					 double baselineJumpsFastW, double baselineJumpsFast, double baselineJumpsSlow, double baselineJumpsLifetime,
					 int baselineLGM, int baselineLGM2State, double baselineLGMProcStd,
					 double baselineLGMX0, double baselineLGMX0p);


QUBFAST_API void qubx_simulate_stimulus_steps(int Nsample, float *samples, double sampling,
					      int Nstep, float *levels, float *durations);
QUBFAST_API void qubx_simulate_stimulus_ramps(int Nsample, float *samples, double sampling,
					      int Nstep, float *levels, float *durations);
  // start ramps with durations[0] == 0

  
#ifdef __cplusplus
}
#endif

#endif
